@@ -13,31 +13,21 @@ import enum
 import json
 import re
 import socket
+import sys
 import unittest
 import unittest.mock
+from collections.abc import KeysView, Sequence, Sized, ValuesView
 from decimal import Decimal
 from pathlib import Path
 from textwrap import dedent
 from types import FunctionType, ModuleType
-from typing import (
-    Any,
-    FrozenSet,
-    KeysView,
-    List,
-    Match,
-    Pattern,
-    Sequence,
-    Set,
-    Sized,
-    Union,
-    ValuesView,
-)
+from typing import Any, ForwardRef, Union
 
 import attr
 import click
 import pytest
 
-from hypothesis import assume
+from hypothesis import HealthCheck, assume, settings
 from hypothesis.errors import InvalidArgument, Unsatisfiable
 from hypothesis.extra import cli, ghostwriter
 from hypothesis.internal.compat import BaseExceptionGroup
@@ -48,7 +38,13 @@ from hypothesis.strategies._internal.lazy import LazyStrategy
 varied_excepts = pytest.mark.parametrize("ex", [(), ValueError, (TypeError, re.error)])
 
 
-def get_test_function(source_code):
+pytestmark = pytest.mark.skipif(
+    settings.get_current_profile_name() == "threading",
+    reason="ghostwriter is not thread safe",
+)
+
+
+def get_test_function(source_code, settings_decorator=lambda fn: fn):
     # A helper function to get the dynamically-defined test function.
     # Note that this also tests that the module is syntatically-valid,
     # AND free from undefined names, import problems, and so on.
@@ -64,7 +60,7 @@ def get_test_function(source_code):
         if k.startswith(("test_", "Test")) and not isinstance(v, ModuleType)
     ]
     assert len(tests) == 1, tests
-    return tests[0]
+    return settings_decorator(tests[0])
 
 
 @pytest.mark.parametrize(
@@ -112,7 +108,7 @@ def test_ghostwriter_exploits_arguments_with_enum_defaults():
         test()
 
 
-def timsort(seq: Sequence[int]) -> List[int]:
+def timsort(seq: Sequence[int]) -> list[int]:
     return sorted(seq)
 
 
@@ -138,7 +134,7 @@ def non_resolvable_arg(x: NotResolvable):
 
 def test_flattens_one_of_repr():
     strat = from_type(Union[int, Sequence[int]])
-    assert repr(strat).count("one_of(") > 1
+    assert repr(strat).count("one_of(") == 2
     assert ghostwriter._valid_syntax_repr(strat)[1].count("one_of(") == 1
 
 
@@ -150,11 +146,11 @@ def takes_values(x: ValuesView[int]) -> None:
     pass
 
 
-def takes_match(x: Match[bytes]) -> None:
+def takes_match(x: re.Match[bytes]) -> None:
     pass
 
 
-def takes_pattern(x: Pattern[str]) -> None:
+def takes_pattern(x: re.Pattern[str]) -> None:
     pass
 
 
@@ -162,7 +158,7 @@ def takes_sized(x: Sized) -> None:
     pass
 
 
-def takes_frozensets(a: FrozenSet[int], b: FrozenSet[int]) -> None:
+def takes_frozensets(a: frozenset[int], b: frozenset[int]) -> None:
     pass
 
 
@@ -223,7 +219,7 @@ def test_ghostwriter_unittest_style(func, ex):
     assert issubclass(get_test_function(source_code), unittest.TestCase)
 
 
-def no_annotations(foo=None, bar=False):
+def no_annotations(foo=None, *, bar=False):
     pass
 
 
@@ -232,7 +228,7 @@ def test_inference_from_defaults_and_none_booleans_reprs_not_just_and_sampled_fr
     assert "@given(foo=st.none(), bar=st.booleans())" in source_code
 
 
-def hopefully_hashable(foo: Set[Decimal]):
+def hopefully_hashable(foo: set[Decimal]):
     pass
 
 
@@ -273,7 +269,7 @@ class A:
         return json.loads(obj)
 
     @staticmethod
-    def static_sorter(seq: Sequence[int]) -> List[int]:
+    def static_sorter(seq: Sequence[int]) -> list[int]:
         return sorted(seq)
 
 
@@ -342,8 +338,9 @@ def test_run_ghostwriter_roundtrip():
         "lambda v: st.lists(v, max_size=2) | st.dictionaries(st.text(), v, max_size=2)"
         ", max_leaves=2)",
     )
+    s = settings(deadline=None, suppress_health_check=[HealthCheck.too_slow])
     try:
-        get_test_function(source_code)()
+        get_test_function(source_code, settings_decorator=s)()
     except (AssertionError, ValueError, BaseExceptionGroup):
         pass
 
@@ -351,7 +348,7 @@ def test_run_ghostwriter_roundtrip():
     source_code = source_code.replace(
         "st.floats()", "st.floats(allow_nan=False, allow_infinity=False)"
     )
-    get_test_function(source_code)()
+    get_test_function(source_code, settings_decorator=s)()
 
 
 @varied_excepts
@@ -462,7 +459,8 @@ def temp_script_file():
             def say_hello():
                 print("Hello world!")
             """
-        )
+        ),
+        encoding="utf-8",
     )
     yield p
     p.unlink()
@@ -482,7 +480,8 @@ def temp_script_file_with_py_function():
             def py():
                 print('A function named "py" has been called')
             """
-        )
+        ),
+        encoding="utf-8",
     )
     yield p
     p.unlink()
@@ -497,7 +496,7 @@ def test_obj_name(temp_script_file, temp_script_file_with_py_function):
     )
     # Windows paths (strings including a "\") should also raise a meaningful UsageError
     with pytest.raises(click.exceptions.UsageError) as e:
-        cli.obj_name("mydirectory\\myscript.py")
+        cli.obj_name(R"mydirectory\myscript.py")
     assert e.match(
         "Remember that the ghostwriter should be passed the name of a module, not a path."
     )
@@ -512,7 +511,7 @@ def test_obj_name(temp_script_file, temp_script_file_with_py_function):
         cli.obj_name(str(temp_script_file))
     assert e.match(
         "Remember that the ghostwriter should be passed the name of a module, not a file."
-        + f"\n\tTry: hypothesis write {temp_script_file.stem}"
+        f"\n\tTry: hypothesis write {temp_script_file.stem}"
     )
     # File names of modules (strings ending in ".py") that define a py function should succeed
     assert isinstance(
@@ -522,3 +521,30 @@ def test_obj_name(temp_script_file, temp_script_file_with_py_function):
 
 def test_gets_public_location_not_impl_location():
     assert ghostwriter._get_module(assume) == "hypothesis"  # not "hypothesis.control"
+
+
+class ForwardRefA:
+    pass
+
+
+@pytest.mark.parametrize(
+    "parameter, type_name",
+    [
+        (ForwardRef("this_ref_does_not_exist"), None),
+        # ForwardRef.evaluate() logic is new in 3.14
+        *(
+            []
+            if sys.version_info[:2] < (3, 14)
+            else [
+                (
+                    ForwardRef("ForwardRefA", owner=A),
+                    ghostwriter._AnnotationData(
+                        "test_ghostwriter.ForwardRefA", {"test_ghostwriter"}
+                    ),
+                )
+            ]
+        ),
+    ],
+)
+def test_parameter_to_annotation(parameter, type_name):
+    assert ghostwriter._parameter_to_annotation(parameter) == type_name
